/*
	DISPLAY5.C

	Copyright Lewis Knox (c) 1989. All rights not expressly granted herein
	are reserved by the author.  User may copy and distribute this file,
	either in its current form or with alterations, as long as he
	does not receive compensation (other than free time on recipient's
	bulletin board system) and as long as he includes this statement
	in the file.  This code may be incorporated into user's programs,
	commercial or otherwise, as long as its authorship is acknowledged
	in the documentation.

	Last revision: 07/14/90

	This program displays a screen file created with scrbin.exe.
	Written in Microsoft Quick C, v. 1.01.

	usage: display filename.ext

*/

 #include <stdio.h>
 #include <dos.h>
 #include <stdlib.h>
 #include <malloc.h>

 #define MINARG  2              /*Required no. of command line arguments.*/
 #define TRUE    0
 #define FALSE  -1
 #define MAXLEN  4000        /*Maximum size of screen file, in bytes.*/
							 /*1 char and 1 attribute for 25 x 80 screen.*/
 #define CGASEG     0xB8000000L  /*Segment address of CGA adapter.*/
 #define MONOSEG    0xB0000000L  /*Segment address of mono adapter.*/
 #define BIOS_INT   0x10     /*Use bios int. 10h*/

 char *err_list[] = {
						"Usage: display infile\n\0",               /*E0*/
						"Error: Unknown video hardware.\n\0",      /*E1*/
						"Error: Cannot find file.\n\0",            /*E2*/
						"Error closing file.\n\0",                 /*E3*/
						"Error determining input file size.\n\0",  /*E4*/
						"Error allocating storage for image.\n\0", /*E5*/
						"Error closing input file.\n\0",           /*E6*/
					 };

 char StartRow = 0;    /*global variables for initial*/
 char StartCol = 0;    /*cursor position.*/

 struct bufparm         /*global struct for image buffer parameters.*/
 {
	long buflen;              /*length of image buffer*/
	unsigned char *bufptr;    /*pointer to image buffer*/
 };

 void cls(void);
 void locate(char, char);
 void get_pos(void);
 int geterr(int);
 long get_vid(void);
 FILE * openinf(char *);
 int closef(FILE *);
 int chek_args(int);
 struct bufparm storepic(char *);
 int show_scr(struct bufparm imgbuf);


 void cls(void)
 {
	union REGS regs;

	regs.h.ah = 0x06;  /*fn 6 of int 10 clears screen*/
	regs.h.al = 0x00;
	regs.h.bh = 0x07;
	regs.h.ch = 0x00;
	regs.h.cl = 0x00;
	regs.h.dh = 0x19;
	regs.h.dl = 0x50;

	int86(BIOS_INT, &regs, &regs);

	regs.h.ah = 0x02;   /*fn 2 of int 10 homes cursor*/
	regs.h.bh = 0x00;
	regs.h.dh = 0x00;
	regs.h.dl = 0x00;

	int86(0x10, &regs, &regs);
 }

 void locate(char row, char col)
 /*set cursor position*/
 {
	union REGS regs;

	regs.h.ah = 0x02;   /*fn 2 of int 10 sets cursor*/
	regs.h.bh = 0x00;
	regs.h.dh = row;
	regs.h.dl = col;

	int86(0x10, &regs, &regs);
 }

 void get_pos(void)
 /*get cursor position*/
 {
	union REGS regs;

	regs.h.ah = 0x03;   /*fn 3 of int 10 gets cursor*/
	regs.h.bh = 0x00;   /*use display page 0*/

	int86(0x10, &regs, &regs);

	StartRow = regs.h.dh;  /*current cursor row returned in dh*/
	StartCol = regs.h.dl;  /*current cursor column return in dl*/
 }


 int get_err(int error_code)
 /*Print error messages*/
 {
	locate(StartRow++,0);   /*place cursor at beginning of line*/
							/*immediately below initial cursor position*/

	if (printf(err_list[error_code]) != 0)  /*if error_code is*/
		exit(TRUE);                       /*invalid, printf returns 0*/
	else
		exit(FALSE);
 }


 int chek_args(int argnum)
 /*Display usage and fail if correct number of args not given.*/
 {
	if (argnum < MINARG)
	{
		get_err(0);     /*"usage: display infile\n"*/
		return(FALSE);
	}
	else
		return(TRUE);
 }


 long get_vid(void)
 {
	union REGS regs;

	regs.h.ah = 0x0F;  /*Function 0Fh of int 10h (get display mode)*/
	int86(BIOS_INT, &regs, &regs);

	/*display mode returned in reg. AL*/
	if ((regs.h.al == 2) || (regs.h.al == 3))
		return(CGASEG);
	else if (regs.h.al == 7)
		return(MONOSEG);
	else
	{
		get_err(1);              /*("Error: unknown video hardware.\n")*/
		return(FALSE);
	}
 }


 FILE * openinf(char *filearg)
 /*
   Open text input file. Return pointer to file stream if successful,
   null pointer on error.
 */
 {
	FILE *fileptr;       /*pointer to input file stream*/

	if ((fileptr = fopen(filearg, "rt")) == NULL) /*open input file*/
	{
		get_err(2);              /*("Cannot find file.\n")*/
		return(fileptr = NULL);  /*Program fails if input file error occurs.*/
	}
	else
		return(fileptr);
 }


 int closef(FILE *fileptr)         /*Close file. Return -1 on error.*/
 {
	int error = 0;      /*error flag*/

	/*fclose returns TRUE if successful.*/
	if ((error = fclose(fileptr)) == TRUE)
		return(TRUE);
	else
	{
		get_err(3);                  /*("Error closing file.\n")*/
		return(FALSE);
	}
 }


 struct bufparm storepic(char *filearg)
 /*
   Allocate buffer of up to 4,000 bytes for image, read image from file
   pointed to by fileptr, and close input file.  Returns structure of
   pointer to buffer and length of buffer.  If error encountered opening
   file, allocating storage, reading file or closing file, return NULL
   pointer.
 */
 {
	int error = 0;                /*error flag*/
	int i = 0;                    /*loop counter*/
	FILE *infile;                 /*Pointer to input file stream.*/
	unsigned char c;              /*Char read from input file.*/
	struct bufparm imgbuf;        /*struct describing image buffer*/

	if ((infile = openinf(filearg)) == NULL)   /*terminate on error*/
	{
		imgbuf.bufptr = NULL;
		imgbuf.buflen = NULL;
		return(imgbuf);
	}
	else
		;

	/*
	  Set storage buffer to size of input file. Set buffer to 4000 bytes
	  if file is bigger than that, but do not terminate. Note that we need
	  to use fileno() because filelength takes the DOS file handle as an
	  argument.
	*/
	if ((imgbuf.buflen = filelength(fileno(infile))) == -1L)
	{
		get_err(4);           /*("Error determining input file size.\n")*/
		imgbuf.bufptr = NULL;                       /*terminate on error*/
		imgbuf.buflen = NULL;
		return(imgbuf);
	}
	else if (imgbuf.buflen > MAXLEN)
		imgbuf.buflen = MAXLEN;
	else
		;

	/*Allocate storage and read file into buffer.*/
	if ((imgbuf.bufptr = malloc((size_t)imgbuf.buflen)) == NULL)
	{
		get_err(5);         /*("Error allocating storage for image.\n")*/
		imgbuf.bufptr = NULL;                      /*terminate on error*/
		imgbuf.buflen = NULL;
		return(imgbuf);
	}
	else
	{                                          /*read file into buffer.*/
		for (i = 0; (i <= imgbuf.buflen) && ((c = getc(infile)) != EOF); i++)
			*(imgbuf.bufptr + i) = c;
	}

	if ((error = closef(infile)) == FALSE)      /*close file*/
	{
		get_err(6);                   /*("Error closing input file.\n")*/
		imgbuf.bufptr = NULL;                      /*terminate on error*/
		imgbuf.buflen = NULL;
		return(imgbuf);
	}
	else
		return(imgbuf);
 }


 int show_scr(struct bufparm imgbuf)
 /*
	Display screen by moving bytes from image buffer to screen.
 */
 {
	int i = 0;          /*loop counters*/
	unsigned char (far * scrptr);    /*pointer to two bytes of video memory.*/
	long result = 0L;      /*error flag*/

	 /*point screen pointer at screen.*/
	 if ((result = get_vid()) != FALSE)
		scrptr = (unsigned char far *) result;
	 else
		return(FALSE);

	locate(80,25);      /*hide cursor*/

	/*move bytes to screen memory*/
	if ((imgbuf.bufptr == NULL) || (imgbuf.buflen == NULL))
		return(FALSE);
	else
	{
		for (i = 0; i <= imgbuf.buflen; i++)
			*(scrptr + i) = *(imgbuf.bufptr + i);
		return(TRUE);
	}
 }

 int main(int argc, char** argv)
 {
	 int error = TRUE;      /*error flag*/

	 /*get initial cursor position*/
	 get_pos();

	 /*enough command line arguments?*/
	 if ((error = chek_args(argc)) == FALSE)
		exit(FALSE);
	 else
		;

	 /*Display file and clear screen when key pressed.*/
	 if ((error = show_scr(storepic(argv[1]))) != FALSE)
	 {
		while (kbhit() == 0)    /*wait for user to hit key*/
			;
		cls();                  /*clear screen*/
		exit(TRUE);
	 }
	 else
	 {
		exit(FALSE);            /*terminate on error*/
	 }
 }

